@@ -4,10 +4,23 @@ from collections import defaultdict
 import numpy as np
 from ase.dft.kpoints import monkhorst_pack
 from TB2J.io_exchange import SpinIO
+from TB2J.Jtensor import decompose_J_tensor, combine_J_tensor
+
+
+def ind_to_indn(ind, n=3):
+    """ index to index. e.g. 1, 2, 4 to 1*3, 1*3+1, 1*3+2, ... 4*3, 4*3+1, 4*3+2
+    :param ind: the input indices
+    :returns: ind3, the output indices
+    """
+    indn = np.repeat(ind, n) * n
+    for i in range(n):
+        indn[i::n] += i
+    return indn
 
 
 class JDownfolder():
-    def __init__(self, JR, Rlist, iM, iL, qmesh):
+
+    def __init__(self, JR, Rlist, iM, iL, qmesh, iso_only=False):
         self.JR = JR
         self.Rlist = Rlist
         self.nR = len(Rlist)
@@ -19,6 +32,9 @@ class JDownfolder():
         self.qmesh = qmesh
         self.qpts = monkhorst_pack(qmesh)
         self.nqpt = len(self.qpts)
+        self.nMn = self.nM*3
+        self.nLn = self.nL*3
+        self.iso_only=iso_only
 
     def get_Jq(self, q):
         Jq = np.zeros(self.JR[0].shape, dtype=complex)
@@ -28,12 +44,15 @@ class JDownfolder():
         return Jq
 
     def get_JR(self):
-        JR_downfolded = np.zeros((self.nR, self.nM, self.nM), dtype=float)
-        Jq_downfolded = np.zeros((self.nqpt, self.nM, self.nM), dtype=complex)
+
+        JR_downfolded = np.zeros((self.nR, self.nMn, self.nMn), dtype=float)
+        Jq_downfolded = np.zeros(
+            (self.nqpt, self.nMn, self.nMn), dtype=complex)
+        self.iMn = ind_to_indn(self.iM, n=3)
+        self.iLn = ind_to_indn(self.iL, n=3)
         for iq, q in enumerate(self.qpts):
             Jq = self.get_Jq(q)
             Jq_downfolded[iq] = self.downfold_oneq(Jq)
-
             for iR, R in enumerate(self.Rlist):
                 phase = np.exp(-2.0j * np.pi * np.dot(q, R))
                 JR_downfolded[iR] += np.real(Jq_downfolded[iq] * phase /
@@ -41,91 +60,93 @@ class JDownfolder():
         return JR_downfolded
 
     def downfold_oneq(self, J):
-        JMM = J[np.ix_(self.iM, self.iM)]
-        JLL = J[np.ix_(self.iL, self.iL)]
-        JLM = J[np.ix_(self.iL, self.iM)]
-        JML = J[np.ix_(self.iM, self.iL)]
+        JMM = J[np.ix_(self.iMn, self.iMn)]
+        JLL = J[np.ix_(self.iLn, self.iLn)]
+        JLM = J[np.ix_(self.iLn, self.iMn)]
+        JML = J[np.ix_(self.iMn, self.iLn)]
         Jn = JMM - JML @ np.linalg.inv(JLL) @ JLM
         return Jn
 
 
 class JDownfolder_pickle():
-    def __init__(self, inpath, metals, ligands, outpath):
-        fname = os.path.join(inpath, 'TB2J.pickle')
-        with open(fname, 'rb') as myfile:
-            self.obj = pickle.load(myfile)
 
-        self.atoms = self.obj['atoms']
-        atoms = self.atoms
-        index_spin = self.obj['index_spin']
-        self.obj['ind_atoms']
-        Jdict = self.obj['exchange_Jdict']
-        nspin = len([x for x in index_spin if x >= 0])
+    def __init__(self, inpath, metals, ligands, outpath, qmesh=[7, 7, 7], iso_only=False):
+        self.exc = SpinIO.load_pickle(path=inpath, fname="TB2J.pickle")
 
+        self.iso_only = (self.exc.dmi_ddict is None) or iso_only
+
+        self.metals = metals
+        self.ligands = ligands
+        self.outpath = outpath
+
+        # read atomic structure
+        self.atoms = self.exc.atoms
+        self.nspin = self.exc.nspin
+        self.qmesh = qmesh
         self.natom = len(self.atoms)
+        self.Rcut = None
+        self._build_atom_index()
+        self._prepare_distance()
+        self._downfold()
 
-        self.magnetic_elements = metals
+    def _build_atom_index(self):
+        self.magnetic_elements = self.metals
         self.iM = []
         self.iL = []
         self.ind_mag_atoms = []
         for i, sym in enumerate(self.atoms.get_chemical_symbols()):
             if sym in self.magnetic_elements:
-                self.iM.append(index_spin[i])
+                self.iM.append(self.exc.index_spin[i])
                 self.ind_mag_atoms.append(i)
-            elif sym in ligands:
-                self.iL.append(index_spin[i])
+            elif sym in self.ligands:
+                self.iL.append(self.exc.index_spin[i])
 
         self.nM = len(self.iM)
         self.nL = len(self.iL)
-        self.nsite = self.nM+self.nL
+        self.nsite = self.nM + self.nL
 
-        # build matrix form of JR
-        JR = defaultdict(lambda: np.zeros(
-            (self.nsite, self.nsite), dtype=float))
-        for key, val in Jdict.items():
-            R, i, j = key
-            JR[R][i, j] = val
-
-        Rlist = list(JR.keys())
-
-        nR = len(Rlist)
-        JR2 = np.zeros((nR, self.nsite, self.nsite), dtype=float)
-        for i, (key, val) in enumerate(JR.items()):
-            JR2[i] = val
-
-        # sum rule
-        iR0 = np.argmin(np.linalg.norm(Rlist, axis=1))
-        for i in range(self.nsite):
-            sum_JRi = np.sum(np.sum(JR2, axis=0)[i])
-            JR2[iR0][i, i] -= sum_JRi
-
-        d = JDownfolder(JR2, Rlist, iM=self.iM, iL=self.iL, qmesh=[7, 7, 7])
+    def _downfold(self):
+        JR2 = self.exc.get_full_Jtensor_for_Rlist(asr=True)
+        d = JDownfolder(JR2, self.exc.Rlist, iM=self.iM,
+                        iL=self.iL, qmesh=self.qmesh, iso_only=self.iso_only)
         Jd = d.get_JR()
 
-        self.Rlist = Rlist
-        self.Rcut = None
-
         self._prepare_distance()
         self._prepare_index_spin()
         self.Jdict = {}
+        if self.iso_only:
+            self.DMIdict = None
+            self.Janidict = None
+        else:
+            self.DMIdict = {}
+            self.Janidict = {}
+
         for iR, R in enumerate(d.Rlist):
             for i, ispin in enumerate(self.index_spin):
                 for j, jspin in enumerate(self.index_spin):
                     if ispin >= 0 and jspin >= 0:
                         if not (tuple(R) == (0, 0, 0) and ispin == jspin):
                             # self interaction.
-                            self.Jdict[(tuple(R), ispin, jspin)
-                                       ] = Jd[iR, ispin, jspin]
+                            J33 = Jd[iR, ispin * 3:ispin * 3 + 3,
+                                     jspin * 3:jspin * 3 + 3]
+                            J, DMI, Jani = decompose_J_tensor(J33)
+                            self.Jdict[(tuple(R), ispin, jspin)] = J
+                            if not self.iso_only:
+                                self.DMIdict[(tuple(R), ispin, jspin)] = DMI
+                                self.Janidict[(tuple(R), ispin, jspin)] = Jani
 
-        io = SpinIO(atoms=atoms,
-                    spinat=self.obj['spinat'],
-                    charges=self.obj['charges'],
+        io = SpinIO(atoms=self.atoms,
+                    spinat=self.exc.spinat,
+                    charges=self.exc.charges,
                     index_spin=self.index_spin,
-                    colinear=True,
+                    colinear=self.iso_only,
                     distance_dict=self.distance_dict,
-                    exchange_Jdict=self.Jdict)
+                    exchange_Jdict=self.Jdict,
+                    dmi_ddict=self.DMIdict,
+                    Jani_dict=self.Janidict,
+                    )
 
-        io.write_all(outpath)
+        io.write_all(self.outpath)
 
     def _prepare_distance(self):
         self.distance_dict = {}
@@ -134,7 +155,7 @@ class JDownfolder_pickle():
         ind_matoms = self.ind_mag_atoms
         for ispin, iatom in enumerate(ind_matoms):
             for jspin, jatom in enumerate(ind_matoms):
-                for R in self.Rlist:
+                for R in self.exc.Rlist:
                     pos_i = self.atoms.get_positions()[iatom]
                     pos_jR = self.atoms.get_positions()[jatom] + np.dot(
                         R, self.atoms.get_cell())
@@ -160,12 +181,16 @@ class JDownfolder_pickle():
                 self.index_spin.append(-1)
 
 
-def test(inpath, outpath, metals, ligands):
-    #path = "/home/hexu/projects/NiCl2/vasp_inputs/TB2J_results"
-    pass
-    #fname = os.path.join(path, "TB2J.pickle")
-    p = JDownfolder_pickle(inpath=inpath, metals=['Ni'], ligands=[
-                           'Cl'], outpath='TB2J_results_downfolded')
+def test():
+    # pass
+    #inpath = "/home/hexu/projects/NiCl2/vasp_inputs/TB2J_results"
+    #inpath = "/home/hexu/projects/TB2J_example/CrI3/TB2J_results"
+    inpath = "/home/hexu/projects/TB2J_projects/NiCl2/TB2J_NiCl/TB2J_results"
+    fname = os.path.join(inpath, "TB2J.pickle")
+    p = JDownfolder_pickle(inpath=inpath,
+                           metals=['Ni'],
+                           ligands=['Cl'],
+                           outpath='TB2J_results_downfolded')
 
 
 if __name__ == "__main__":
