@@ -37,6 +37,7 @@ def gen_exchange(path,
                  output_path='TB2J_results',
                  wannier_type="wannier90",
                  qspace=False,
+                 orb_decomposition=False,
                  description=''):
     try:
         fname=os.path.join(path, posfile)
@@ -74,7 +75,8 @@ def gen_exchange(path,
             print("Reading Banddownfolder hamiltonian: spin up.")
             tbmodel_up = MyTB.load_banddownfolder(path=path,
                                                   prefix=prefix_up,
-                                                  atoms=atoms,                                                   nls=False)
+                                                  atoms=atoms,
+                                                  nls=False)
             print("Reading Banddownfolder hamiltonian: spin down.")
             tbmodel_dn = MyTB.load_banddownfolder(path=path,
                                                   prefix=prefix_dn,
@@ -225,7 +227,9 @@ Warning: Please check if the noise level of Wannier function Hamiltonian to make
                                ne=ne,
                                np=np,
                                use_cache=use_cache,
-                               description=description)
+                               description=description,
+                               orb_decomposition=orb_decomposition,
+                               )
         print("\n")
         exchange.run(path=output_path)
         print(
@@ -246,6 +250,7 @@ def gen_exchange_siesta(fdf_fname,
                         np=1,
                         use_cache=False,
                         output_path='TB2J_results',
+                        orb_decomposition=False,
                         description=''):
 
     try:
@@ -260,11 +265,12 @@ def gen_exchange_siesta(fdf_fname,
             f"sisl version is {sisl.__version__}, but should be larger than 0.10.0."
         )
     fdf = sisl.get_sile(fdf_fname)
+    geom = fdf.read_geometry()
     H = fdf.read_hamiltonian()
     if H.spin.is_colinear:
         print("Reading Siesta hamiltonian: colinear spin.")
-        tbmodel_up = SislWrapper(H, spin=0)
-        tbmodel_dn = SislWrapper(H, spin=1)
+        tbmodel_up = SislWrapper(H, spin=0, geom=geom)
+        tbmodel_dn = SislWrapper(H, spin=1, geom=geom)
         basis = dict(zip(tbmodel_up.orbs, list(range(tbmodel_up.norb))))
         print("Starting to calculate exchange.")
         description = f""" Input from collinear Siesta data.
@@ -282,10 +288,6 @@ def gen_exchange_siesta(fdf_fname,
             emin=emin,
             emax=emax,
             nz=nz,
-            #height=height,
-            #nz1=nz1,
-            #nz2=nz2,
-            #nz3=nz3,
             exclude_orbs=exclude_orbs,
             Rcut=Rcut,
             ne=ne,
@@ -298,10 +300,10 @@ def gen_exchange_siesta(fdf_fname,
             f"All calculation finsihed. The results are in {output_path} directory."
         )
 
-    elif H.spin.is_colinear:
+    elif H.spin.is_colinear and False:
         print(
-            "Reading Siesta hamiltonian: colinear spin. Treat as non-colinear")
-        tbmodel = SislWrapper(H, spin='merge')
+            "Reading Siesta hamiltonian: colinear spin. Treat as non-colinear. For testing only.")
+        tbmodel = SislWrapper(H, spin='merge', geom=geom)
         basis = dict(zip(tbmodel.orbs, list(range(tbmodel.nbasis))))
         print("Starting to calculate exchange.")
         description = f""" Input from collinear Siesta data.
@@ -323,7 +325,8 @@ def gen_exchange_siesta(fdf_fname,
                                ne=ne,
                                np=np,
                                use_cache=use_cache,
-                               description=description)
+                               description=description,
+                               orb_decomposition=orb_decomposition)
         exchange.run(path=output_path)
         print("\n")
         print(
@@ -333,7 +336,7 @@ def gen_exchange_siesta(fdf_fname,
     elif H.spin.is_spinorbit or H.spin.is_noncolinear:
 
         print("Reading Siesta hamiltonian: non-colinear spin.")
-        tbmodel = SislWrapper(H, spin=None)
+        tbmodel = SislWrapper(H, spin=None, geom=geom)
         basis = dict(zip(tbmodel.orbs, list(range(tbmodel.nbasis))))
         print("Starting to calculate exchange.")
         description = f""" Input from non-collinear Siesta data.
@@ -358,7 +361,9 @@ Warning: The DMI component parallel to the spin orientation, the Jani which has
                                ne=ne,
                                np=np,
                                use_cache=use_cache,
-                               description=description)
+                               description=description,
+                               orb_decomposition= orb_decomposition
+                               )
         exchange.run(path=output_path)
         print("\n")
         print(
